//*****************************************
// AT mega328P-PU(28pin 300mil SDIP) one IN-14 Clock Program
// CPU Clock:8MHz
//   iq[Yf[^@WȂBIV[^(8MHz)ɂj
//
// File Name:nixie.c
//
// ̐ؑ֎@Xɑւdg݂Ă邪AjLV[ǂ̓
// ؂ւɐOサČBȂ̂Ō݂͎̂悤Ȏg͂ĂȂB
// 񂾂ɏ̂͂قǌÂ炭͖B
//
//  2017/11/18 New  Taishi
//*****************************************

#include	<avr/pgmspace.h>
#include	<avr/io.h>

#include	"one_in14_clock.h"
#include	"nixie.h"

// **********************************************************
// *********************** NIXE TUBE ************************
// **********************************************************
//

// **********************************************************
//@jLV[ǂ͂O`XƍE_\
//
//@@\locSCTCUCV̂Srbĝabcōs
//@@@V@U@T@S
//@@@k@k@k@k@||@O
//@@@k@k@k@g@||@P
//@@@k@k@g@k@||@Q
//@@@k@k@g@g@||@R
//@@@k@g@k@k@||@S
//@@@k@g@k@g@||@T
//@@@k@g@g@k@||@U
//@@@k@g@g@g@||@V
//@@@g@k@k@k@||@W
//@@@g@k@k@g@||@X
//@@@g@k@g@k@||@
//@@@g@k@g@g@||@
//@@@g@g@k@k@||@
//@@@g@g@k@g@||@
//@@@g@g@g@k@||@
//@@@g@g@g@g@||@
//
//@@_\ihbgj
//@@@oaOFgœ_Akŏ
//
//@@E_\ihbgj
//@@@ocPFgœ_Akŏ
//
// **********************************************************

#include	"nixie.h"

static	unsigned char	nixie_number[2] = {0xf0,0xf0};
static	unsigned char	nixie_dot;
static	unsigned char	dynamic_nixie = 0;
static	unsigned char	dynamic_disp_ratio0 = DISP_RATE_MAX;
static	unsigned char	dynamic_disp_count = 0;

#define		NIXE_DATA_MASK		0x0d	// PD 1,4--7 Data Mask
#define		NIXE_SELECT_CLR		0xe0	// PC0 -- 5 Clear
#define		NIXE_NUMBER_OFF		0xf0	// PD4--7:'H'
#define		NIXE_NUMBER_MASK	0x0f	// PD4--7:'L'ɂ鎞
#define		NIXE_DOT_RIGHT		0x02	// PD1 
#define		NIXE_DOT_LEFT		0x01	// PB0 
//******************************
//  Nixe\@
// *****************************
void	init_nixe(void)
{

	PORTD &= NIXE_DATA_MASK;					// A_}XN
	PORTD |= NIXE_NUMBER_OFF;					// \Ȃ
	PORTD &= ~(NIXE_DOT_RIGHT);					// E_OFF
	PORTB &= ~(NIXE_DOT_LEFT);					// _OFF
	DDRD |= ((1 << DDD1) | (1 << DDD4) | (1 << DDD5) | (1 << DDD6) | (1 << DDD7));	// Port-D Bit1,4567 : Output
	DDRC |= ((1 << DDC0) | (1 << DDC1) | (1 << DDC2) | (1 << DDC3) | (1 << DDC4));	// Port-C Bit01234: Output
	DDRB |= (1 << DDB0);															// Port-B Bit0: Output

// \A_ f[^
	nixie_dot = NIXE_DOT_OFF;					// _NA
	nixie_number[0] = NIXE_NUMBER_OFF;			// NAi\Ȃj
	nixie_number[1] = NIXE_NUMBER_OFF;			// NAi\Ȃj
	dynamic_disp_ratio0 = DISP_RATE_MAX;		// \䗦

	dynamic_nixie = 0;

}
//******************************
// 1̃jLV[Ǖ\
// sQ̊荞݂ŌĂ΂
//  d͖TO`Ȃ邪A邭Ȃ
//  AhbgAEhbgŕ\
//******************************
void	proc_dynamic_nixie(void)
{
	static	unsigned char	path=0;						// 0:number, 1:left dot, 2:right dot
	unsigned char	dd;

	if(dynamic_nixie == 1){
		PORTD |= NIXE_NUMBER_OFF;						// \Ȃ
		PORTB &= ~NIXE_DOT_LEFT;						// _nee
		PORTD &= ~NIXE_DOT_RIGHT;						// E_nee
		if(path == 0){
			PORTD &= ~(NIXE_DOT_RIGHT);					// E_nee
			PORTB &= ~(NIXE_DOT_LEFT);					// _nee
			if(dynamic_disp_ratio0 < dynamic_disp_count){
				PORTD &= (nixie_number[1] | NIXE_NUMBER_MASK);	// ̂Õrbg𐶂A|[gɃZbg
			}else{
				PORTD &= (nixie_number[0] | NIXE_NUMBER_MASK);	// ̂Õrbg𐶂A|[gɃZbg
			}
			dynamic_disp_count++;
			if(DISP_RATE_MAX <dynamic_disp_count){
				dynamic_disp_count = 0;
			}
		}else if(path == 1){
			dd = nixie_dot;
			if((dd & NIXE_DOT_L_ON) != 0){				// _nm̎
				PORTB |= NIXE_DOT_LEFT;					// _nm
			}else{
				PORTB &= ~NIXE_DOT_LEFT;				// _nee
			}
		}else if(path == 2){
			dd = nixie_dot;
			if((dd & NIXE_DOT_R_ON) != 0){				// E_nm̎
				PORTD |= NIXE_DOT_RIGHT;				// E_nm
			}else{
				PORTD &= ~NIXE_DOT_RIGHT;				// E_nee
			}
		}else{
			PORTD |= NIXE_NUMBER_OFF;					// \Ȃ
			PORTD &= ~(NIXE_DOT_RIGHT);					// E_nee
			PORTB &= ~(NIXE_DOT_LEFT);					// _nee
		}

		path++;
		if(3 <= path){
			path = 0;
		}

	}
}

//******************************
// Dynamic Nixie display Active/Non active
//  on : 1= Active, other=Non Active
// *****************************
void	dynamic_nixie_active(unsigned char on)
{
	dynamic_nixie = on;
}
//******************************
// Nixie _\
//  on : NIXE_DOT_R_ON, NIXE_DOT_L_ON, NIXE_DOT_L_ON | NIXE_DOT_R_ON, NIXE_DOT_OFF
// *****************************
void	disp_dynamic_nixie_dot(unsigned char on)
{
	nixie_dot = on;
}
//******************************
// Nixie \
//  O: O`XAPTiPT͋󔒁jOԖڑ
//  P: O`XAPTiPT͋󔒁jPԖڑ
// *****************************
void	disp_dynamic_nixie_number(unsigned char n0,unsigned char n1)
{
	nixie_number[0] = ((n0 << 4) & 0xf0);
	nixie_number[1] = ((n1 << 4) & 0xf0);
}
//******************************
// Nixie \䗦iOԖڂ̕\䗦j
//  : O`DISP_RATE_MAX
//   DISP_RATE_MAXŃtɂOԖڂ̐\
//   DISP_RATE_MAX-1łOԖڂDISP_RATE_MAX-1APԖڂPŕ\
// *****************************
void	disp_dynamic_nixie_ratio(char ratio)
{
	dynamic_disp_ratio0 = (unsigned char)ratio;
}
