//*****************************************
// AT mega328P-PU(28pin 300mil SDIP) one IN-14 Clock Program
// CPU Clock:8MHz
//   iq[Yf[^@WȂBIV[^(8MHz)ɂj
//
// File Name:timer.c
//
//  2017/11/18 New  Taishi
//*****************************************
#include	<avr/interrupt.h>
#include	<avr/io.h>

#include	"one_in14_clock.h"
#include	"nixie.h"
#include	"ad_conv.h"

#include	"timer.h"

// **********************************************************
// ********************** Soft Timer ************************
// **********************************************************

volatile unsigned int	wait_i;
volatile unsigned int	wait_j;
volatile unsigned char	wait_us;
// ************************************
// Delay 2uS
// t: uS (max 255)
// CPU Clock:8.000MHz
//  while ߂ for ŎԂ킹Ă
// ************************************
void delay_2us(unsigned char t)
{
	unsigned char	i;

	for(i=0;i<t;i++){
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
	}
}
//***************************
// Wait
//  tm : Wait Time(ms) max. 8000
// CPU Clock:1MHz
//***************************
void	delay_ms(int tm)
{

	for(wait_i = 0; wait_i < tm; wait_i++){
		delay_2us(500/2);
		delay_2us(500/2);
	}
}

// **********************************************************
// ************************ Timer 1 *************************
// **********************************************************

int		pulse_out;
int		pulse_period;
int		pulse_unit;
static	unsigned char	tim1_unit[5]={
	(1 << CS10),					// 1/1
	(1 << CS11),					// 1/8
	((1 << CS11) | (1 << CS10)),	// 1/64
	(1 << CS12),					// 1/256
	((1 << CS12) | (1 << CS10)),	// 1/1024
};
//*****************************************
// Timer1 close
//*****************************************
void	timer1_close(void)
{
	TCCR1B = 0x00;														// Timer1 Stop

	DDRB |= (1 << DDB1);												// Port-B Bit1 : Output
	PORTB &= ~(1 << PB1);												// Port-B Bit1 = Low
	TCNT1 = 0;															// Timer1 Count Set to ZERO
	TCCR1A = 0;															//
	TCCR1B = 0;															//

}

//*****************************************
// Pulse ON
// \ timer1  OCR1A,ICR1̃WX^̓ZbgĂKv
//*****************************************
void	pulse_on(void)
{
	unsigned char	ud;

	TCCR1A = ( (1 << COM1A1) | (0 << COM1A0) | (1 << WGM11) | (0 << WGM10) );	// PWM TOP=ICR1, OC1A:BOTTOM=H Match=L, 
	ud = TCCR1B;										// read Timer1 Prescaler
	ud &= ~( (1 << CS12) | (1 << CS11) | (1 << CS10) );	// Timer1 Prescaler Clear
	ud |= tim1_unit[pulse_unit];						// Timer1 Prescaler data Set
	TCCR1B = ud;										// Timer1 Prescaler Set
}
//*****************************************
// Pulse OFF
//*****************************************
void	pulse_off(void)
{
	TCCR1A = ( (0 << COM1A1) | (0 << COM1A0) | (1 << WGM11) | (0 << WGM10) );	// Standerd Port TOP=ICR1, OC1A:BOTTOM=H Match=L, 
	TCCR1B &= ~( (1 << CS12) | (1 << CS11) | (1 << CS10) );						// Timer1 Prescaler Stop
	TCNT1 = 0;																	// Timer1 Count Set to ZERO
}
//*****************************************
// Set Timer1 ON Time
//*****************************************
void	set_timer1_ontime(void)
{
	unsigned int	ud;

	if(PULSE_OUT_MAX <= pulse_out){
		ud = PULSE_OUT_MAX;
	}else if(pulse_out <= PULSE_OUT_MIN){
		ud = PULSE_OUT_MIN;
	}else{
		ud = (unsigned int)pulse_out;
	}
	OCR1A  = ud;								// ON Time ( 1=125nS : 8MHz)
}
//*****************************************
// Set Timer1 Period
//*****************************************
void	set_timer1_period(void)
{
	unsigned int	ud;

	ud = (unsigned int)pulse_period;
	ICR1  = ud;								// Period Time ( 1=125nS : 8MHz)
}
//*****************************************
// Timer1 Initial
//  PD4(OC1B): Pulse Out for High Volt Generator
//*****************************************
void	timer1_init(void)
{
	TCCR1B = 0x00;														// Timer1 Stop
	PRR &= ~( 1 << PRTIM1 );											// Timer1 ғ

	PORTB &= ~(1 << PB1);												// Port-B Bit1=Low
	DDRB |= (1 << DDB1);												// Port-B Bit1 : Output

	pulse_out = PULSE_OUT_DEFAULT;										// pXnm
	pulse_period = PULSE_PERIOD_DEFAULT;								// pX
	pulse_unit = PULSE_UNIT_DEFAULT;									// PpXP

	TCNT1 = 0;															// Timer1 Count Set to ZERO
	TCCR1A = ( (1 << COM1A1) | (0 << COM1A0) | (1 << WGM11) | (0 << WGM10) );	// PWM TOP=ICR1, OC1A:BOTTOM=H Match=L, 
	TCCR1B = ( (1 << WGM13) | (1 << WGM12) );							// A Comper Match

	set_timer1_period();												// 8MHz / 1000 = 8kHz(125uS)
	OCR1A  = (unsigned int)pulse_out;									// On Time ( 1=125nS)

}

//***************************
// Timer1 Comper Match A Interrupt
//
//  ̏125uS Ԋu
// Ă΂Ȃ͂
//***************************
ISR	(TIMER1_COMPA_vect)	
{
}
//***************************
// Timer1 Comper Match B Interrupt
//
//  ̏125uS Ԋu
// Ă΂Ȃ͂
//***************************
ISR	(TIMER1_COMPB_vect)	
{
}
// **********************************************************
// ************************ Timer 2 *************************
// **********************************************************
static unsigned char	tim_05ms;				// 10mSPʂJE^[
static unsigned char	tim_10ms_for_1s;		// PbPʂJE^[
unsigned char	tim_10ms_ad;					// `^cf[^擾pPOrPʃ^C}[
unsigned char	tim_10ms_sw;					// XCb`͗pPOrPʃ^C}[
unsigned char	tim_10ms_sw_length;				// XCb`pԂPOrPʃ^C}[@|P
unsigned char	tim_10ms_sw_continue;			// XCb`pAPOrPʃ^C}[@|P
unsigned char	tim_1s_no_key_timeup;			// L[͂Ȃ̃^CAbvp
unsigned char	count_1s_disp;					// t[̂PbJE^[
unsigned char	tim_10ms_blink;					// \uNpPOrPʃ^C}[
unsigned char	tim_10ms_nixie;					// jLV[\̃^C~OpPOrPʃ^C}[
//*****************************************
// Timer2 Initial
//  1mS 荞݂ɐݒ肷
//*****************************************
void	timer2_init(void)
{
	TCCR2B = 0;											// Timer Stop
	PRR &= ~( 1 << PRTIM2 );							// Timer2 ғ
	TCNT2 = 0;											// Timer2 Count Set to ZERO
	TCCR2A = ((1 << WGM21) | (0 << WGM20));				// A Comper Match, Dont Output Timer2 Signal
	OCR2A  = COUNT_05MS;								// 1mS 8MHz/32=250kHz
	TCCR2B = D_TCCR2B;									// Timer Start (8MHz/32= 250kHz, 4uS)
	TIMSK2 |= (1 << OCIE2A);							// A Comper Match Interrupt Enable

	tim_05ms = 1;										// 10mSPʂJE^[
	tim_10ms_for_1s = 0;								// PbPʂJE^[
	tim_10ms_ad = 0;									// `^cf[^擾pPOrPʃ^C}[
	tim_10ms_sw = 0;									// XCb`͗pPOrPʃ^C}[
	tim_10ms_sw_length = 0;								// XCb`pԂPOrPʃ^C}[@|P
	tim_10ms_sw_continue = 0;							// XCb`pAPOrPʃ^C}[@|P
	tim_1s_no_key_timeup = 0;							// L[͂Ȃ̃^CAbvp
	count_1s_disp = 0;									// t[̂PbJE^[
	tim_10ms_blink = 0;									// \uNpPOrPʃ^C}[
	tim_10ms_nixie = 0;									// jLV[\̃^C~OpPOrPʃ^C}[

}
//*****************************************
// Timer2 Close
//  Timer2 ̎gp߂
//*****************************************
void	timer2_close(void)
{
	TCCR2B = 0;											// Timer Stop
	TCNT2 = 0;											// Timer2 Count Set to ZERO
	TCCR2A = 0;											// Dont Output Timer2 Signal
	TIMSK2 = 0;											// Interrupt Disable

}
//***************************
// Timer2 Comper Match A Interrupt
//
//  0.5mS 
//***************************
ISR	(TIMER2_COMPA_vect)	
{
	tim_05ms--;
	if(tim_05ms == 0){
		tim_05ms = 20;
		tim_10ms_for_1s++;
		if(100 <= tim_10ms_for_1s){				// Pbo
			tim_10ms_for_1s = 0;
			count_1s_disp++;					// t[̂PbJE^[
		}

		if(tim_10ms_ad != 0){
			tim_10ms_ad--;						// `^cf[^擾pPOrPʃ^C}[@|P
		}

		if(tim_10ms_sw != 0){
			tim_10ms_sw--;
		}

		if(tim_10ms_sw_continue != 0){
			tim_10ms_sw_continue--;				// XCb`pAPOrPʃ^C}[@|P
		}

		if(tim_10ms_sw_length != 0){
			tim_10ms_sw_length--;				// XCb`pԂPOrPʃ^C}[@|P
		}

		if(tim_10ms_blink != 0){
			tim_10ms_blink--;
		}

		if(tim_10ms_nixie != 0){
			tim_10ms_nixie--;
		}

#if	AD_NO == 6
		ad6_set_data();							// A/D6  Set A/D data to Data Buffer
#else
 #if	AD_NO == 5
		ad5_set_data();							// A/D5  Set A/D data to Data Buffer
 #else
		ad0_set_data();							// A/D0  Set A/D data to Data Buffer
 #endif
#endif

	}
	proc_dynamic_nixie();
}

//***************************
// ԃL[͂̔f
//@ on:1 = L[LA@0= ^CAbṽ`FbN
//  ߂l@O@^CAbvĂȂ
//@@O@^CAbv
//***************************
unsigned char	proc_no_key_timeup(unsigned char on)
{
	unsigned char	r=0;

	if (on == 1){
		tim_1s_no_key_timeup = TIM_DISP_RETURN;		// ^CAbv̎Ԑݒ
	}else{
		if(tim_1s_no_key_timeup == 0){				// ^CAbv
			r = 0xff;
		}
	}
	return(r);
}
